import os
from dataclasses import dataclass

@dataclass
class Wheel:
    """
    {distribution_name}-{version}-{pyversion}-{ABItag}-{platform}.whl
    """

    full_name: str
    distribution_name: str
    version: str
    pyversion: str
    ABItag: str
    platform: str

    @classmethod
    def from_path(cls, path: str):
        whl_name, ext = os.path.splitext(path)
        if ext != ".whl":
            raise Exception(f"Expected .whl file, got {ext}")
        whl = whl_name.split("/")[-1]

        splits = whl.split("-")
        w = cls(f"{whl}.whl", *splits)
        w.distribution_name = w.distribution_name.replace("_", "-")
        return w



import os
import threading
from functools import partial
from http.server import HTTPServer as BaseHTTPServer
from http.server import SimpleHTTPRequestHandler
from logging import getLogger

from github import Repository
from pi_conf import Config
from platformdirs import site_cache_dir

from dpypi import ROOT_DIR, cfg
from dpypi.github_connection import GithubConnection, GithubConnections
from dpypi.web_utils import write_index_html
from dpypi.wheel import Wheel

log = getLogger(__name__)
## add stdout handler
import logging
import sys

log.addHandler(logging.StreamHandler(sys.stdout))
log.setLevel(logging.DEBUG)


class HandlerConfig(Config):
    web_dir: str
    artifact_dir: str
    html_dir: str
    template_dir: str


connections = GithubConnections(cfg)


class HTTPHandler(SimpleHTTPRequestHandler):
    """This handler uses server.base_path instead of always using os.getcwd()"""

    def __init__(self, config: HandlerConfig, *args, **kwargs):
        self.config: HandlerConfig = config
        super().__init__(*args, **kwargs)

    @staticmethod
    def _split_path(path: str) -> tuple[str, str]:
        ext = os.path.splitext(path)[1]
        if ext:
            distribution = path.split("/")[-2]
        else:
            distribution = path.removesuffix("/").split("/")[-1]
        return distribution, ext

    def _make_distribution_html(
        self,
        repo: Repository.Repository,
        connection: GithubConnection,
    ) -> str:
        distribution = repo.name

        log.debug(f"_make_distribution_html: distribution: {distribution}, {self.path}")
        html_path = os.path.join(self.config.html_dir, distribution, "index.html")
        if os.path.exists(html_path):
            log.debug(f"html_path cache exists at {html_path}")
            return html_path

        assets = connection.list_release_assets(repo)
        assets = [a for a in assets if a.name.endswith(".whl")]
        write_index_html(
            project=distribution,
            release_assets=assets,
            html_path=html_path,
            artifact_dir=self.config.artifact_dir,
            template_dir=self.config.template_dir,
        )
        log.debug(f"wrote {html_path}")
        return f"/{html_path}"

    def do_GET(self):
        """
        Serve a GET request.
        /shutdown : will shutdown the server
        /simple/<distribution>/ : will serve the distribution's html page
        /simple/<distribution>/<artifact> : will serve the artifact
        """
        if self.path == "/shutdown":
            self.send_response(200)
            self.end_headers()
            threading.Thread(target=self.server.shutdown, daemon=True).start()
            return

        distribution, ext = self._split_path(self.path)
        connection = connections.repo_2_connection.get(distribution)
        log.debug(
            f"do_GET: distribution: {distribution} found={connection is not None} path={self.path}"
        )
        if not connection:
            super().do_GET()
            return
        repo = connection.g.get_repo(f"{connection.name}/{distribution}")
        if not ext:
            self.path = self._make_distribution_html(repo, connection)
        elif ext == ".whl":
            w = Wheel.from_path(self.path)

            asset = connection.get_release_asset(
                repo=repo,
                release=w.version,
                asset_name=w.full_name,
                dest_folder=os.path.join(self.config.artifact_dir, w.distribution_name),
            )
            self.path = asset.local_path
        log.debug(f"~do_GET(): {self.path}")
        super().do_GET()


def main():
    port = cfg.get("port", 8083)
    base_path = cfg.get("base_path", "")
    conf = HandlerConfig(
        artifact_dir=cfg.get("artifact_dir", "web/artifacts"),
        html_dir=cfg.get("html_dir", "web/html"),
        template_dir=cfg.get("template_dir", f"{ROOT_DIR}/templates"),
    )
    http_handler = partial(HTTPHandler, conf)
    httpd = BaseHTTPServer((base_path, port), http_handler)
    log.debug(f"HTTPServer: address{httpd.server_address}")
    httpd.serve_forever()


if __name__ == "__main__":
    main()



import os
from dataclasses import dataclass, field
from io import BytesIO, StringIO
from logging import getLogger

import requests
from github import Auth, Github
from github.GitRelease import GitRelease
from github.GitReleaseAsset import GitReleaseAsset
from github.Repository import Repository
from pi_conf import Config

log = getLogger(__name__)


@dataclass
class IndexConfig(Config):
    name: str
    uri: str
    repos: list[str] = None
    access_token: str = None


@dataclass
class ReleaseAsset(GitReleaseAsset):
    ## Hack to add a local_path attribute to GitReleaseAsset for type hinting
    local_path: str = None


@dataclass
class GithubConnections:
    config: Config
    connections: dict[str, "GithubConnection"] = field(default_factory=dict)
    _repo_2_connection: dict = None

    def __post_init__(self):
        index_cfg: IndexConfig
        redact = set(["access_token"])
        for index_cfg in self.config.index:
            for k in index_cfg:
                cfg = index_cfg.copy()
                if k in redact:
                    cfg[k] = "*****"
                log.debug(f"{k}={cfg[k]}")
            gc = GithubConnection(index_cfg)
            self.connections[index_cfg.name] = gc

        self.projects = {}

    @property
    def repo_2_connection(self) -> dict[str, "GithubConnection"]:
        if self._repo_2_connection is None:
            self._repo_2_connection = self._make_repos()
        return self._repo_2_connection

    def _make_repos(self) -> dict:
        d: dict[str, GithubConnection] = {}
        for gc in self.connections.values():
            repo_filter = set(gc.config.repos) if "repos" in gc.config else None
            for repo in gc.g.get_user().get_repos():
                if repo_filter and repo.name not in repo_filter:
                    continue
                d[repo.name] = gc
        return d


@dataclass
class GithubConnection:
    config: IndexConfig

    _g: Github = None
    _auth = None

    @property
    def name(self) -> str:
        return self.config.name

    @property
    def uri(self) -> str:
        return self.config.uri

    @property
    def repo_names(self) -> list[str]:
        return self.config.repos

    @property
    def auth(self) -> Auth.Token:
        if self._auth is None:
            self._auth = Auth.Token(self.config.access_token)
        return self._auth

    @property
    def g(self) -> Github:
        if self._g is None:
            self._g = Github(auth=self.auth)

        return self._g

    def list_release_assets(
        self,
        repo: Repository,
    ) -> list[GitReleaseAsset]:
        """
        Get all assets for all releases in a repository
        args:
            repo: Repository.Repository
        returns:
            list[GitReleaseAsset.GitReleaseAsset]
        """
        assets = []
        for release in repo.get_releases():
            for asset in release.get_assets():
                assets.append(asset)
        return assets

    def get_release_asset(
        self,
        repo: Repository,
        release: str | GitRelease,
        asset_name: str,
        dest_folder: str | StringIO | BytesIO,
        overwrite: bool = False,
    ) -> ReleaseAsset:
        asset_name = asset_name.split("/")[-1]
        if not isinstance(release, GitRelease):
            try:
                release = repo.get_release(release)
            except Exception as e:
                raise Exception(f"Release {release} not found") from e
        if release is None:
            raise Exception(f"Release {release} not found")

        for asset in release.assets:
            if asset.name != asset_name:
                continue
            session = requests.Session()
            headers = {
                "Authorization": "token " + self.auth.token,
                "Accept": "application/octet-stream",
            }
            os.makedirs(dest_folder, exist_ok=True)
            dest = os.path.join(dest_folder, asset.name)
            asset.local_path = dest
            if os.path.exists(dest) and not overwrite:
                log.debug(f"File {dest} already exists, skipping")
                return asset

            response = session.get(asset.url, stream=True, headers=headers)
            with open(dest, "wb") as f:
                for chunk in response.iter_content(1024 * 1024):
                    f.write(chunk)
            return asset
        return None



import os

import bs4
from github import GitReleaseAsset


def write_index_html(
    project: str,
    release_assets: list[GitReleaseAsset.GitReleaseAsset],
    html_path: str,
    artifact_dir: str,
    template_dir: str,
) -> str:
    """
    Write an index.html file for a project's distribution.
    args:
        project: the name of the project
        release_assets: a list of release assets
        html_path: the path to the index.html
        artifact_dir: the directory where the release assets are stored
        template_dir: the directory containing the template
    returns:
    str: the path to the index.html file
    """
    template_path = os.path.join(template_dir, "distribution.html")
    with open(template_path) as f:
        soup = bs4.BeautifulSoup(f, features="html.parser")
    div = soup.find("div", {"class": "revisions"})
    for asset in release_assets:
        name = asset.browser_download_url.split("/")[-1]
        tag = soup.new_tag("a", href=f"/{artifact_dir}/{project}/{asset.name}")
        tag.string = name
        div.append(tag)
    os.makedirs(os.path.dirname(html_path), exist_ok=True)

    with open(html_path, "w") as f:
        f.write(soup.prettify())
    return html_path
